#!/bin/lua

-- (c) 2007 Robert Shingledecker
-- .dfmext Tool
-- icon bar by mikshaw

require("fltk")
dofile("/etc/init.d/functions5.lua")

images={}
dir="/usr/share/dfm/icons/" -- ending slash is vital
total_images=100 -- with no limit, I get segfaults (maybe just corrupted image files?)

dirlist = readDir(dir)
for k,v in ipairs(dirlist) do
  if k > total_images then break end
  if string.find(v,"%.xpm$") then
  table.insert(images,v)
  end
end

function get_image_data(self)
 dfmext[2] = dir..self:tooltip()
 icon_btn:image(fltk:Fl_XPM_Image((dfmext[2])))
 icon_btn:redraw()
end

function tokenize(dfmextLine)
   local data = {}
   for hit,nohit in string.gmatch(dfmextLine,"([^;]*)") do
      if hit ~= "" then
         data[#data+1] = hit
      end
   end
   return data
end

function packData()
   dfmext[1] = ext_input:value()
   dfmext[3] = cmd_input:value()
   if dfmext[1] == "" then
      fltk.fl_alert("E' richiesta l'estensione del file!")
      return
   end
   if dfmext[3] == "" then dfmext[3] = "!0!" end
   newdata = dfmext[1]..";"..dfmext[2]..";"..dfmext[3]
   if selector:value() == 0 then
      selector:add(newdata)
      saveNeeded = true
   else
      selector:insert(selector:value(),newdata)
      saveNeeded = true
   end
   clearData()
   selector:deselect()
end

function clearData()
   ext_input:value("")
   icon_btn:image(fltk:Fl_XPM_Image((defaultIcon)))
   icon_btn:redraw()
   cmd_input:value("!0!")
   dfmext[2]=defaultIcon
end

function saveDfmext()
   local dfmextFile,err = io.open(".dfmext","w")
   if not dfmextFile then
      print("Impossibile aprire il file "..dfmextItems..": "..err)
      os.exit(1)
   end
   for i = 1, selector:size() do
      if selector:text(i) ~= "" then
      dfmextFile:write(selector:text(i).."\n")
      end
   end
   dfmextFile:close()
   saveNeeded = false
end

ww = 560
wh = 350
saveNeeded = false
defaultIcon = "/usr/share/dfm/icons/prg.xpm"
dfmext = {"",defaultIcon,"!0!"}

w = fltk:Fl_Double_Window(ww,wh,"DSL strumento .dfmext")
w:callback(
function(w)
   if saveNeeded == true then 
      if fltk.fl_choice("Vuoi salvare il nuovo file .dfmext?","No","Si",NULL) == 1 then saveDfmext() end
   end
   os.exit(0)
end)

selector = fltk:Fl_Hold_Browser(0,0,ww-82,225)
selector:callback(
function(selector)
   selected_item = selector:text(selector:value())
   dfmext = tokenize(selected_item)
   ext_input:value(dfmext[1])
   icon_btn:image(fltk:Fl_XPM_Image((dfmext[2])))
   cmd_input:value(dfmext[3])
   editGroup:redraw()
end)
if not selector:load("/opt/.dfmext") then
   print("impossibile aprire /opt/.dfmext")
   os.exit(1)
end
-- remove last line if empty
if selector:text(selector:size()) == "" then
   selector:remove(selector:size())
end

editBox = fltk:Fl_Box(5,226,ww-90,85)
editBox:box(fltk.FL_EMBOSSED_BOX)
editGroup = fltk:Fl_Group(0,228,ww-82,wh-37)
ext_input = fltk:Fl_Input(140,240,150,25,"Nome file/Estensione:")
ext_input:value(dfmext[1])
icon_btn = fltk:Fl_Button(395,245,48,48)
icon_btn:image(fltk:Fl_XPM_Image((dfmext[2])))
cmd_input = fltk:Fl_Input(140,275,220,25,"Comando:")
cmd_input:value(dfmext[3])
fltk:Fl_End() -- End editGroup Layout

btnGroup = fltk:Fl_Group(0,wh-37,ww-82,wh-37)
editBtn = fltk:Fl_Button(110,wh-32,50,25,"&Edita")
editBtn:shortcut('e')
editBtn:callback(
function(editBtn)
   if selector:value() == 0 then
      fltk.fl_alert("Devi selezionare un elemento da editare!")
   else
      savePosition = selector:value()
      selector:remove(selector:value())
      selector:value(savePosition)
      packData()
      saveNeeded = true
   end
end)

delBtn = fltk:Fl_Button(170,wh-32,50,25,"&Elimina")
delBtn:shortcut('d')
delBtn:callback(
function(delBtn)
   if selector:value() == 0 then
      fltk.fl_alert("Devi selezionare un elemento da eliminare!")
   else
      selector:remove(selector:value())
      clearData()
      selector:deselect()
      saveNeeded = true
   end
end)

clearBtn = fltk:Fl_Button(230,wh-32,50,25,"&Pulisci")
clearBtn:shortcut('c')
clearBtn:callback(clearData)

addOk = fltk:Fl_Button(290,wh-32,50,25,"&Aggiungi")
addOk:shortcut('a')
addOk:callback(packData)

helpBtn = fltk:Fl_Button(350, wh-32, 50, 25, "Aiuto")
helpBtn:shortcut('h')
helpBtn:callback(
function(helpBtn)
   fltk.fl_message(
"Seleziona un elemento dfmext o posizionati x aggiungere un nuovo elemento.\n"..
"Edita o elimina elementi. X aggiungerne uno nella posizione desiderata\n"..
"Pulisci i campi, premi aggiungi.")
end)

fltk:Fl_End() -- End btnGroup Layout

iconGroup = fltk:Fl_Group(421,0,ww,350)
Fl:set_boxtype(fltk.FL_UP_BOX,fltk.FL_THIN_UP_BOX)
Fl:set_boxtype(fltk.FL_DOWN_BOX,fltk.FL_THIN_DOWN_BOX)
scroll=fltk:Fl_Scroll(480,5,72,350)
pack=fltk:Fl_Pack(480,5,48,350)
pack:spacing(1)
icon={}; butt={}
scroll:add(pack)
for i=1,table.getn(images) do
 icon[i]=fltk:Fl_XPM_Image(dir..images[i])
 if icon[i]:w()==32 and icon[i]:h()==32 then -- 32x32 only
   butt[i]=fltk:Fl_Button(0,0,48,48)
   butt[i]:image(icon[i])
   butt[i]:tooltip(images[i])
   butt[i]:callback(get_image_data)
   pack:add(butt[i])
 else icon[i]:uncache() -- uncache images not 32x32
 end
end
fltk:Fl_End() -- End iconGroup Layout

selector:position(0)

w:show()
Fl:run()
