#!/bin/lua
-- Launches applications under X
-- by Daniel Plata Lorenzo "MakodFilu"
-- Thanks to mikshaw and Robert Shingledecker for the input and ideas

require("fltk")
version="1.1.2"
max_history=10
history_file=os.getenv("HOME").."/.flrun/history"
assoc_file=os.getenv("HOME").."/.flrun/assoc"
ww=250  --window width
wh=100  --   "   height
bw=60   --button width
bh=25   --   "   height
text={
w="flRun v"..version,
input="Lancia Applicazione",
cancel="&Annulla",ok="&Ok",
browse="&Sfoglia",
choose="Scegli Applicazione"}
tip={
cancel="Annulla lancio",
ok="Lancia applicazione",
browse="Sfoglia x un comando"}

function existfile(file)
  local f=io.open(file,"r")
  if f then f:close() return(1) end
end

function readtable(file,separator)
  f=io.open(file,"r")
  t={}
  if separator then
    for line in f:lines() do
      i,i=line:find(separator)
      if i then table.insert(t,{line:sub(1,i-1),line:sub(i+1)}) end
    end
  else
    for line in f:lines() do table.insert(t,line) end
  end
  f:close()
  return(t)
end

function historysort(self,match)
  t={}
  for i=1,#self,1 do
    if self[i]~=match then table.insert(t,self[i]) end
  end
  table.insert(t,match)
  return(t)
end 

function writetable(file,t,limit)
  io.output(io.open(file,"w+"))
  for i=math.max(1,#t-(limit-1)),#t,1 do io.write(t[i]..'\n') end
  io.close()
end

function exec_command(command)
  if command:find("%.%a*$") and existfile(assoc_file) then
    t=readtable(assoc_file,":")
    ext=command:sub(command:find("%.%a*$"))
    for i=1,#t,1 do 
      if ext == '.'..t[i][1] then command=t[i][2]..' '..command end
    end -- Redefine as app+doc
  end   -- if .ext is not recognized, launch anyway
  os.execute(command..' 2>/dev/null &')
  if not persist then os.exit(0) end
end

command=""
for i=1,#arg,1 do
  if arg[i]=="--help" or arg[i]=="--version" then
    print ("flRun v"..version.." lancia applicazioni in X\n")
    print "Usage flrun [options] [file options]"
    print "   file options     Launcia il file direttamente, senza box d dialogo"
    print "   --persist        Dialog persistance"
    print "   --preload file   carica file come comando d default "
    print "   --notips         Disabilita aiuto" 
    print "   --version,--help Mostra questa guida\n"
    os.exit(0)
  elseif arg[i]=="--notips" then tip={}
  elseif arg[i]=="--persist" then persist=1
  elseif arg[i]=="--preload" then preload=1
  elseif command=="" then command=arg[i]
  else command=command..' '..arg[i]
  end
end
if not preload and command~="" then exec_command(command) end

w=fltk:Fl_Window(ww,wh,text.w)
input=fltk:Fl_Input_Choice(10,30,ww-20,bh,text.input)
input:align(fltk.FL_ALIGN_TOP)
if existfile(history_file) then
  history=readtable(history_file)
  for i=1,#history,1 do input:add(history[i]) end
  input:value(history[#history]) -- Show last feed command
end
if preload then input:value(command) end

ok=fltk:Fl_Return_Button(ww-3*bw*1.2,wh-35,bw,bh,text.ok)
ok:tooltip(tip.ok)
ok:callback(
function(ok)
  if input:value() == "" then io.write(string.char(7)) io.flush()
  else
    if history then history=historysort(history,input:value())
    else history={input:value()}
    end
    writetable(history_file,history,max_history)
    exec_command(input:value())
  end
end)

cancel=fltk:Fl_Button(ww-2*bw*1.2,wh-35,bw,bh,text.cancel)
cancel:tooltip(tip.cancel)
cancel:callback(
function(cancel)
  os.exit(0)
end)

browse=fltk:Fl_Button(ww-1*bw*1.2,wh-35,bw,bh,text.browse)
browse:tooltip(tip.browse)
browse:callback(
function(browse)
  local txt = fltk.fl_file_chooser(text.choose,"*","")
  if txt then input:add(txt) end
end)
w:show()
Fl:run()
