#!/bin/bash
# Original script by Robert Shingledecker for John Andrews DSL
# (c) Robert Shingledecker 2003,2004,2005
# A simple script to save/restore configs, directories, etc defined by the user
# in the file .filetool.lst
# Added ideas from WDef for invalid device check and removal of bfe password upon failure

. /etc/init.d/dsl-functions

CMDLINE="$(cat /proc/cmdline)"

if [ -z $1 ] ; then 
  echo "Uso: filetool.sh {backup|restore}"
  echo -n "Premi invio x continuare:" ; read ans
  exit 1
fi

#Get the TARGET name from argument 3 or /opt/.backup_device
if [ -z $3 ]; then
  TARGET="$(cat /opt/.backup_device 2>/dev/null)"
else
  TARGET="$3"
fi


TARGET="${TARGET#/dev/}"
DEVICE="${TARGET%%/*}"
FULLPATH="${TARGET#$DEVICE/}"
[ "$FULLPATH" = "$DEVICE" ] && FULLPATH=""

#Exit if no device specified or if device is not a valid block device
#If qemu allow special virtual drives of type f
[ "x$DEVICE" = x ] && exit 1
if [ ! -b "/dev/$DEVICE" ]; then
  if checkbootparam "qemu"; then
    [ ! -f "/cdrom/qemu/$DEVICE" ] && exit 1
  else
    exit 1
  fi
fi
 
find_mountpoint $DEVICE

if [ -z "$MOUNTPOINT" ]; then
  echo "Periferica $DEVICE non valida"
  echo -n "Premi invio x continuare:" ; read ans
  exit 1
fi

if [ $MOUNTED == "no" ]; then
   sudo mount $MOUNTPOINT
   if [ "$?" != 0 ]; then
      echo "Impossibile montare la periferica $DEVICE"
      echo -n "Premi invio x continuare:" ; read ans
      exit 1
   fi
fi


echo "$TARGET" > /opt/.backup_device

blowfish(){
KEY=$(cat /etc/sysconfig/bfe)
cat << EOD | /usr/bin/bcrypt -c "$MOUNTPOINT"/"$FULLPATH"/$1 2>/dev/null
"$KEY"
"$KEY"
EOD
if [ "$?" != 0 ]; then failed; fi
sync
}

clean_up(){
if [ $MOUNTED == "no" ]; then
	sudo umount $MOUNTPOINT
fi

# Only store device name if backup/restore successful
[ $1 -eq 0 ] && echo "$TARGET" > /opt/.backup_device

# Remove bfe password if decryption fails
[ $1 -eq 98 ] && rm -f /etc/sysconfig/bfe

sync
exit $1
}

failed(){
echo "${RED}failed.${NORMAL}"
clean_up 98
}

trap failed SIGTERM

if [ $1 == "backup" ] ; then
  if [ -z $2 ]; then
    sudo tar -C / -T /opt/.filetool.lst -X /opt/.xfiletool.lst  -czvf "$MOUNTPOINT"/"$FULLPATH"/backup.tar.gz
    echo -n "Premi invio x continuare:" ; read ans
  else
    echo -n "${BLUE}Sto creando il backup su ${YELLOW}$MOUNTPOINT/$FULLPATH ${BLUE}montato sulla periferica ${MAGENTA}/dev/$DEVICE ${NORMAL}"
    sudo tar -C / -T /opt/.filetool.lst -X /opt/.xfiletool.lst  -czf "$MOUNTPOINT"/"$FULLPATH"/backup.tar.gz  2>/tmp/backup_status &
    rotdash $!
    sync
    [ -s /tmp/backup_status ] && exit 1
    if [ -f /etc/sysconfig/bfe ]; then
       echo -n "sto criptando .. "
       blowfish backup.tar.gz
    fi
  fi
  echo "${BLUE}Done.${NORMAL}"
  clean_up 0
fi

if [ $1 == "restore" ] ; then
  if [ -f /etc/sysconfig/bfe ]; then
     TARGETFILE="backup.tar.gz.bfe"
  else
     TARGETFILE="backup.tar.gz"
  fi
  if [ ! -f $MOUNTPOINT/$FULLPATH/$TARGETFILE ] ; then
     if [ $MOUNTED == "no" ]; then
      sudo umount $MOUNTPOINT
     fi
  fi
  if [ -z $2 ]; then
    tar -C / -zxvf "$MOUNTPOINT"/"$FULLPATH"/backup.tar.gz
    echo -n "Premi invio x continuare:" ; read ans
  else
    echo -n "${BLUE}Sto ripristinando il file di backup da ${YELLOW}$MOUNTPOINT/$FULLPATH ${BLUE}montato sulla periferica ${MAGENTA}/dev/$DEVICE ${NORMAL}"
    if [ -f /etc/sysconfig/bfe ]; then
       echo -n "sto decriptando .. "
       blowfish "$TARGETFILE"
    fi
    tar -C / -zxf "$MOUNTPOINT"/"$FULLPATH"/backup.tar.gz 2>/dev/null &
    rotdash $!
    echo "${BLUE}Done.${NORMAL}"
  fi
  clean_up 0
fi
echo "Non capisco i parametri della riga d comando : $1"
echo "Usage: filetool.sh {backup|restore}"
echo -n "Premi invio x continuare:" ; read ans
exit 1
